	.text
	.globl	__aeabi_unwind_cpp_pr0
__aeabi_unwind_cpp_pr0:
	.globl	__aeabi_unwind_cpp_pr1
__aeabi_unwind_cpp_pr1:
	.globl	__aeabi_unwind_cpp_pr2
__aeabi_unwind_cpp_pr2:
	.globl	__aeabi_unwind_cpp_pr3
__aeabi_unwind_cpp_pr3:
