/*
 * arch/mips/klibc/archsocket.h
 */

#ifndef _KLIBC_ARCHSOCKET_H
#define _KLIBC_ARCHSOCKET_H

#ifndef SOCK_STREAM
# define SOCK_DGRAM     1
# define SOCK_STREAM    2
# define SOCK_RAW       3
# define SOCK_RDM       4
# define SOCK_SEQPACKET 5
# define SOCK_PACKET    10
#endif

#endif /* _KLIBC_ARCHSOCKET_H */
