/*
 * sys/syscall.h
 *
 * Generic system call interface macros
 */
#ifndef _SYS_SYSCALL_H
#define _SYS_SYSCALL_H

#include <errno.h>
#include <sys/types.h>
#include <asm/unistd.h>

#endif				/* _SYS_SYSCALL_H */
