/*
 * bits32/limits.h
 */

#ifndef _BITSIZE_LIMITS_H
#define _BITSIZE_LIMITS_H

#define LONG_BIT	32

#define LONG_MIN	(-2147483647L-1)
#define LONG_MAX	2147483647L
#define ULONG_MAX	4294967295UL

#endif				/* _BITSIZE_LIMITS_H */
