/*
 * bits64/stdintlimits.h
 */

#ifndef _BITSIZE_STDINTLIMITS_H
#define _BITSIZE_STDINTLIMITS_H

#define INT_FAST16_MIN	INT64_MIN
#define INT_FAST32_MIN	INT64_MIN
#define INT_FAST16_MAX	INT64_MAX
#define INT_FAST32_MAX	INT64_MAX
#define UINT_FAST16_MAX UINT64_MAX
#define UINT_FAST32_MAX UINT64_MAX

#define INTPTR_MIN	INT64_MIN
#define INTPTR_MAX	INT64_MAX
#define UINTPTR_MAX	UINT64_MAX

#define PTRDIFF_MIN	INT64_MIN
#define PTRDIFF_MAX	INT64_MAX

#endif				/* _BITSIZE_STDINTLIMITS_H */
