/*
 * endian.h
 */

#ifndef _ENDIAN_H
#define _ENDIAN_H

#include <klibc/endian.h>

#define LITTLE_ENDIAN	__LITTLE_ENDIAN
#define BIG_ENDIAN	__BIG_ENDIAN
#define PDP_ENDIAN	__PDP_ENDIAN
#define BYTE_ORDER	__BYTE_ORDER

#endif				/* _ENDIAN_H */
