/*
 * arch/mips/include/sgidefs.h
 */

/* Some ABI constants */

#ifndef _SGIDEFS_H
#define _SGIDEFS_H

#define _MIPS_ISA_MIPS1 1
#define _MIPS_ISA_MIPS2 2
#define _MIPS_ISA_MIPS3 3
#define _MIPS_ISA_MIPS4 4
#define _MIPS_ISA_MIPS5 5

#define _MIPS_SIM_ABI32         1
#define _MIPS_SIM_NABI32        2
#define _MIPS_SIM_ABI64         3

#endif				/* _SGIDEFS_H */
